
/*
 * $Id: ContentCfg.cc,v 1.5 1999/09/26 21:51:19 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Rnd.h"

#include "MimeSym.h"
#include "ContentSym.h"
#include "EmbedContMdl.h"

#include "ContentCfg.h"


ContentCfg::ContentCfg(int anId):
	theSize(0), theChbRatio(-1), theEmbedCont(0), theId(anId) {
}

void ContentCfg::configure(const ContentSym *cfg) {
	Assert(cfg);

	if (MimeSym *mime = cfg->mime()) {
		theMimeType = mime->mimeType();
	}
	theSize = cfg->size();
	theObjLifeCycle.configure(cfg->objLifeCycle());
	cfg->cachable(theChbRatio);

	if (cfg->hasEmbed()) {
		theEmbedCont = new EmbedContMdl;
		theEmbedCont->configure(cfg);
	}
}

double ContentCfg::compLocalContrib(ContentCfg *cc) const {
	if (cc->id() == id())
		return 1.0;

	if (theEmbedCont)
		return theEmbedCont->compLocalContrib(cc);

	return 0.0;
}

void ContentCfg::noteGlobalContrib(ContentCfg *cc, double cc_prob, double contrib) {
	if (cc->id() == id())
		return;

	if (theEmbedCont)
		theEmbedCont->noteGlobalContrib(cc, cc_prob, contrib);
}
