
/*
 * $Id: ContentMgr.cc,v 1.1 1999/09/26 21:51:20 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */


#include "polygraph.h"

#include "ErrorMgr.h"
#include "ContentCfg.h"
#include "ContentMgr.h"

#include "ContentSym.h"

#include "polyErrors.h"


ContentMgr TheContentMgr;


ContentMgr::ContentMgr() {
}

ContentMgr::~ContentMgr() {
	while (theCfgs.count()) delete theCfgs.pop();
	while (theSyms.count()) delete theSyms.pop();
}

// this method can be called re-entrantly!
ContentCfg *ContentMgr::get(const ContentSym *cs) {
	Assert(cs);

	int idx;
	for (idx = 0; idx < theSyms.count(); ++idx) {
		if (cs->equal(*theSyms[idx]))
			return theCfgs[idx];
	}

	// no match found
	ContentCfg *cfg = new ContentCfg(idx+1);
	theCfgs.append(cfg);
	theSyms.append((ContentSym*)cs->dup());

	// configure after append to facilitate reentrant calls
	cfg->configure(cs);
	return cfg;
}

ContentCfg *ContentMgr::get(int id) {
	int idx = id - 1;
	Assert(idx >= 0 && theCfgs.count() > 0);
	if (idx >= theCfgs.count()) {
		ReportError(errWrongContentId);
		idx = 0;
	}
	return theCfgs[idx];
}

