
/*
 * $Id: SrvConnMgr.cc,v 1.12.4.1 1999/10/05 19:58:54 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "ErrorMgr.h"
#include "SrvConnMgr.h"

#include "polyBcastChannels.h"
#include "polyErrors.h"


SrvConnMgr::SrvConnMgr(User *aUser): theUser(aUser), theAcceptLmt(-1) {
	Assert(theUser);
}

void SrvConnMgr::acceptLmt(int aLmt) {
	theAcceptLmt = aLmt;
}

// return false iff fatal error
bool SrvConnMgr::accept(Socket &s) {
	// note: negative accept limit means no limit
	for (int i = theAcceptLmt; i; --i) {
		bool fatal = false;
		Connection *conn = TheConnFarm.get();
		if (conn->accept(s, theSockOpt, fatal)) {
			opened(conn);
			conn->theRd.start(this); // wait for data
		} else {
			// note: no close() for an un-opened conn
			TheConnFarm.put(conn);
			return !fatal;
		}
	}
	return true;
}

// this gets called for an idle conn and on the freshly accepted conn
void SrvConnMgr::noteReadReady(int fd) {
	Connection *conn = TheConnIdx[fd];
	Assert(conn);
	conn->theRd.stop(this);

	conn->read(); // we do the first read, not a xaction

	if (conn->bad()) { // bad read error
		put(conn);
		Broadcast(TheXactErrChannel, (Xaction*)0);
	} else
	if (conn->exhausted()) { // closed conn with no buffered data
		if (conn->useCnt() > 0) {
			conn->closeKind(ConnCloseStat::ckIdleForeign);
		} else {
			ReportError(errNoUseClose);
			Broadcast(TheXactErrChannel, (Xaction*)0);
		}
		put(conn);
	} else {
		conn->use();
		theUser->noteConnReady(conn); // more requests or close
	}
}

void SrvConnMgr::putIdle(Connection *conn) {
	Assert(!conn->theRd.theReserv && !conn->theWr.theReserv);
	// there may be more requests waiting
	if (conn->theRdBuf.contSize()) {
		conn->use();
		theUser->noteConnReady(conn);
	} else {
		// start waiting for more requests and for close
		conn->theRd.start(this);
	}
}
