
/*
 * $Id: StatPhase.h,v 1.12.6.2 1999/10/14 16:56:25 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_STAT_PHASE_H
#define PG_STAT_PHASE_H

#include "Goal.h"
#include "ErrorStat.h"
#include "StatIntvl.h"
#include "StatPhaseRec.h"

class StatPhaseMgr;
class PhaseSym;

// stats phase is a named [long] interval during which
// stats are accumulated; the phase goal may not be [just] time based
// the stats are logged when instructed by the manager
// when phase goal is reached stat phase terminates notifying the manager

class StatPhase: public StatIntvl {
	public:
		StatPhase();
		virtual ~StatPhase();

		void configure(const PhaseSym *ps, const StatPhase *prevPh);
		const char *name() const { return theRec.theName; }
		bool used() const { return wasUsed; }
		double loadFactorBeg() const;
		double loadFactorEnd() const;

		Time duration() const;
		int xactCnt() const { return theRec.complCnt(); }
		BigSize fillSz() const { return theRec.fillSz(); }
		int errCnt() const { return theRec.theXactErrCnt; }
		double errRatio() const { return theRec.errRatio(); }

		void statsLogged(bool are);
		void name(const String &aName);

		bool varyLoad() const { return theLoadFactK; }
		double loadFactor(Time lastFactTm, int lastFactCnt) const;

		void start(StatPhaseMgr *aMgr);
		void flush();

		virtual void wakeUp(const Alarm &alarm);

		virtual void noteConnEvent(BcastChannel *ch, const Connection *c);
		virtual void noteXactEvent(BcastChannel *ch, const Xaction *x);
		virtual void noteErrNoEvent(BcastChannel *ch, int errNo);

		void noteSockRead(Size sz) { theRec.theSockRdSzH.record(sz); }
		void noteSockWrite(Size sz) { theRec.theSockWrSzH.record(sz); }

		void reportCfg(ostream &os) const;

	protected:
		bool checkpoint();
		void stop();
		void report() const;

	protected:
		StatPhaseMgr *theMgr;
		StatPhaseRec theRec;
		ErrorStat theErrs;     // not a part of PhaseRec, logged separately
		Goal theGoal;          // configurable GoalRec with reached()

		bool wasUsed;          // was listening for events
		bool logStats;         // log stats when flushed

		double theLoadFactBeg;
		double theLoadFactEnd;
		double theLoadFactK;		
};


#endif
