
/*
 * $Id: PglRateSym.cc,v 1.4 1999/09/07 17:47:47 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "PglNumSym.h"
#include "PglTimeSym.h"
#include "PglRateSym.h"

#include "gadgets.h"

RateSym::RateSym(double aCount, Time anInterval): ExpressionSym("rate"),
	theCount(aCount), theInterval(anInterval) {
}

RateSym::RateSym(double aVal): ExpressionSym("rate") {
	val(aVal);
}

bool RateSym::isA(const String &type) const {
	return ExpressionSym::isA(type) ||
		type == "Rate" || type == "rate";
}

SynSym *RateSym::clone(const String &type) const {
	if (isA(type))
		return new RateSym(theCount, theInterval);
	return ExpressionSym::clone(type);
}

double RateSym::val() const {
	return Ratio(theCount, theInterval.secd());
}

void RateSym::val(double aVal) {
	theCount = aVal;
	theInterval = Time::Sec(1);
}

ExpressionSym *RateSym::unOper(const String &op) const {
	if (*op == '+')
		return new RateSym(+theCount, theInterval);
	else
	if (*op == '-')
		return new RateSym(-theCount, theInterval);
	else
		return ExpressionSym::unOper(op);
}

ExpressionSym *RateSym::bnOper(const String &op, const SynSym &s) const {
	if (op.len() != 1)
		return ExpressionSym::bnOper(op, s);

	if (RateSym *rs = (RateSym*)s.clone("rate"))
		return operRR(op, rs, s);

	if (TimeSym *ts = (TimeSym*)s.clone("time"))
		return operRT(op, ts, s);

	if (NumSym *ns = (NumSym*)s.clone("float"))
		return operRN(op, ns, s);

	return ExpressionSym::bnOper(op, s);
}

// rate op rate
ExpressionSym *RateSym::operRR(const String &op, RateSym *rs, const SynSym &s) const {
	double otherVal = rs->val();
	delete rs;

	if (*op == '/')
		checkDenom(otherVal);

	switch (*op) {
		case '+': return new RateSym(val() + otherVal);
		case '-': return new RateSym(val() - otherVal);
		case '/': return new NumSym(val() / otherVal);
		default:  return ExpressionSym::bnOper(op, s);
	}
}

ExpressionSym *RateSym::operRT(const String &op, TimeSym *ts, const SynSym &s) const {
	Time otherInterval = ts->val();
	delete ts;

	switch (*op) {
		case '*': return new NumSym(theCount*(otherInterval/theInterval));
		default: return ExpressionSym::bnOper(op, s);
	}
}

ExpressionSym *RateSym::operRN(const String &op, NumSym *ns, const SynSym &s) const {
	double otherCount = ns->val();
	delete ns;
	
	if (*op == '/')
		checkDenom(otherCount);

	switch (*op) {
		case '*': return new RateSym(theCount*otherCount, theInterval);
		case '/': return new RateSym(theCount/otherCount, theInterval);
		default: return ExpressionSym::bnOper(op, s);
	}
}

ostream &RateSym::print(ostream &os, const String &) const {
	return os << val() << "/sec";
}
