
/*
 * $Id: PglRecSym.cc,v 1.9 1999/09/28 20:44:52 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "PglCtx.h"
#include "PglRec.h"
#include "PglRecSym.h"

#include "PglBoolSym.h"
#include "PglIntSym.h"
#include "PglNumSym.h"
#include "PglDistrSym.h"
#include "PglTimeSym.h"


RecSym::~RecSym() {
	delete theRec;
}

bool RecSym::isA(const String &type) const {
	return ExpressionSym::isA(type) || type == "Rec";
}

SynSym *RecSym::clone(const String &type) const {
	Assert(theRec);

	if (isA(type))
		return new RecSym(this->type(), theRec->clone());
	else
		return ExpressionSym::clone(type);
}

ostream &RecSym::print(ostream &os, const String &pfx) const {
	os << '{' << endl;
	if (theRec)
		theRec->report(os, pfx+"    ");
	os << pfx << '}';
	return os;
}

bool RecSym::getBool(const String &name, bool &set) const {
	SynSymTblItem *bi = 0;
	Assert(theRec);
	Assert(theRec->find(name, bi));
	if (bi->sym())
		set = ((const BoolSym&)bi->sym()->cast("bool")).val();
	return bi->sym();
}

bool RecSym::getInt(const String &name, int &n) const {
	SynSymTblItem *ii = 0;
	Assert(theRec);
	Assert(theRec->find(name, ii));
	if (ii->sym())
		n = ((const IntSym&)ii->sym()->cast("int")).val();
	return ii->sym();
}

bool RecSym::getDouble(const String &name, double &n) const {
	SynSymTblItem *di = 0;
	Assert(theRec);
	Assert(theRec->find(name, di));
	if (di->sym())
		n = ((const NumSym&)di->sym()->cast("float")).val();
	return di->sym();
}

Time RecSym::getTime(const String &name) const {
	SynSymTblItem *ti = 0;
	Assert(theRec);
	Assert(theRec->find(name, ti));
	return ti->sym() ?
		((const TimeSym&)ti->sym()->cast("time")).val() : Time();
}

RndDistr *RecSym::getDistr(const String &name) const {
	const String typeName = "Distr";
	SynSymTblItem *di = 0;
	Assert(theRec);
	Assert(theRec->find(name, di));
	return di->sym() ?
		((const DistrSym&)di->sym()->cast(typeName)).val() : 0;
}

