
/*
 * $Id: polyOpts.cc,v 1.13.2.2 1999/10/16 05:58:46 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <string.h>

#include "Select.h"
#include "Poll.h"
#include "rndDistrs.h"
#include "opts.h"
#include "polyOpts.h"
#include "polyVersion.h"
#include "gadgets.h"


/* FileScanOpt */

FileScanOpt::FileScanOpt(OptGrp *aGrp, const char *aName, const char *aDescr, FileScanner *def):
	Opt(aGrp, aName, aDescr), theVal(def) {
}

bool FileScanOpt::parse(const char *name, const char *val) {

	delete theVal; theVal = 0;

	const String v = val;
	if (v == "select")
		theVal = new Select();
	else
	if (v == "poll")
		theVal = new Poll();
	else
		cerr << "unknown file scanner name (use `poll' or `select') for the `" << name << "' option; got: " << v << endl;

	return theVal;
}

void FileScanOpt::report(ostream &os) const {
	os << (theVal ? theVal->name() : "<none>");
}


/* DumpFlagsOpt */

DumpFlagsOpt::DumpFlagsOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	ListOpt(aGrp, aName, aDescr) {
	memset(theFlags, 0, sizeof(theFlags));
}

bool DumpFlagsOpt::addItem(const String &item) {
	// which part of the message to dump
	DumpFlags part = dumpAny;
	if (item.str("hdr"))
		part = dumpHdr;
	else
	if (item.str("body"))
		part = dumpBody;

	// what kind of messages to dump
	if (item.str("req"))
		setFlag(dumpReq, part);
	else
	if (item.str("rep"))
		setFlag(dumpRep, part);
	else
	if (item.str("err"))
		setFlag(dumpErr, part);
	else
	if (part != dumpAny || item == "All") {
		setFlag(dumpReq, part);
		setFlag(dumpRep, part);
		setFlag(dumpErr, part);
	} else {
		cerr << "unknown flag `" << item << "' for the `" << name() << "' option" << endl;
		return false;
	}

	return true;
}

void DumpFlagsOpt::setFlag(DumpFlags type, DumpFlags part) {
	theFlags[type] |= part;
	theFlags[0] |= part;
}

const char *DumpFlagsOpt::dumpPartStr(DumpFlags type) const {
	if (theFlags[type] == dumpHdr)
		return "-hdr";
	if (theFlags[type] == dumpBody)
		return "-body";
	return "";
}

void DumpFlagsOpt::report(ostream &os) const {
	if (!theFlags[0]) {
		os << "<none>";
		return;
	}
		
	if (theFlags[dumpReq])
		os << "req," << dumpPartStr(dumpReq);
	if (theFlags[dumpRep])
		os << "rep," << dumpPartStr(dumpRep);
	if (theFlags[dumpErr])
		os << "err" << dumpPartStr(dumpErr);
}


/* HostTypeOpt */

HostTypeOpt::HostTypeOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr) {
}

bool HostTypeOpt::parse(const char *, const char *) {
	Assert(theCmdLine);
	cerr
		<< "vesrion " << PolyVersion()
		<< " built on " << CONFIG_HOST_TYPE
		<< " at " << PolyCompTime() << ' ' << PolyCompDate()
		<< endl;
	exit(0);
}

void HostTypeOpt::report(ostream &os) const {
	os << CONFIG_HOST_TYPE;
}


/* VersionOpt */

VersionOpt::VersionOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr) {
}

bool VersionOpt::parse(const char *, const char *) {
	Assert(theCmdLine);
	cerr << PolyLetterhd() << endl;
	exit(0);
}

void VersionOpt::report(ostream &os) const {
	os << PolyVersion();
}
