
/*
 * $Id: shutdown.cc,v 1.10 1999/09/17 23:54:32 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Socket.h"
#include "Select.h"
#include "String.h"
#include "AlarmClock.h"
#include "LogComment.h"
#include "PolyOLog.h"
#include "Progress.h"
#include "SharedOpts.h"
#include "StatPhaseMgr.h"
#include "shutdown.h"

#include "globals.h"


static String TheReason = 0;
static int TheSignal = -1;

static bool DoShutdown = false; // ShutdownReason() has been called
static bool InShutdown = false; // Shutdown() has started, has not finished
static bool IsClean = false;    // Shutdown() has finished

// check various shutdown conditions
bool ShutdownNow() {
	if (DoShutdown)
		return true; // already decided
	else
	if (Socket::Level() >= Socket::TheMaxLevel) {
		Comment << "active file descriptors: " << Socket::Level() << "; limit: " << Socket::TheMaxLevel << endc;
		ShutdownReason("dangerously low on file descriptors");
	} else
	if (TheSignal >= 0) {
		Comment << "noticed shutdown signal (" << TheSignal << ")" << endc;
		ShutdownReason("got shutdown signal");
	} else
		return false;

	return DoShutdown = true;
}

bool ShutdownRequested() {
	return DoShutdown;
}

void ShutdownSignal(int s) {
	TheSignal = s;
	clog << "got shutdown signal (" << s << ")" << endl;
}

void ShutdownReason(const String &reason) {
	if (reason)
		TheReason = reason;
	DoShutdown = true;
}

void Shutdown() {
	if (IsClean || InShutdown)
		return; // nothing to do; was here already?!

	DoShutdown = InShutdown = true;

	Clock::Update(false);

	Comment(2) << "got " << TheProgress.xacts()
		<< " xactions and " << TheProgress.errs() << " errors" << endc;
	if (TheReason)
		Comment << "shutdown reason: " << TheReason << endc;

	// cleanup
	TheStatPhaseMgr.forceFlush();
	TheOLog.close();

	IsClean = true;
	InShutdown = false;
}
