
/*
 * $Id: BigSize.cc,v 1.7 1999/09/07 17:48:24 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xstd.h"

#include <iostream.h>
#include <limits.h>
#include <math.h>

#include "Assert.h"
#include "BigSize.h"


/* static creators */

BigSize BigSize::MB(int n) {
	const int factor = INT_MAX/(1024*1024); // MBs in a chunk
	Assert(factor > 0);
	BigSize sz;
	sz.theCnt = n / factor; // chunks in n
	sz.theAcc = (n % factor) * (1024*1024); // leftovers
	return sz;
}

/* dynamic methods */

BigSize::BigSize(int bytes): theCnt(0), theAcc(bytes) {
}

BigSize &BigSize::operator +=(const BigSize &bsz) {
	theCnt += bsz.theCnt;
	(*this) += bsz.theAcc;
	return *this;
}

BigSize &BigSize::operator +=(int sz) {
	if (sz < 0)
		return (*this) -= -sz;

	const int space = INT_MAX - theAcc;
	if (space <= sz) { // overflow
		theAcc = sz - space;
		theCnt++;
	} else {
		theAcc += sz;
	}
	return *this;
}

BigSize &BigSize::operator -=(const BigSize &bsz) {
	theCnt -= bsz.theCnt;
	(*this) -= bsz.theAcc;
	return *this;
}

BigSize &BigSize::operator -=(int sz) {
	if (sz <= 0)
		return (*this) += -sz;

	const int space = theAcc - INT_MIN;

	if (space < sz) { // underflow
		theAcc += INT_MAX - sz;
		theCnt--;
	} else {
		theAcc -= sz;
	}
	return *this;
}

BigSize &BigSize::operator *=(double f) {
	const double d = byted() * f;
	theCnt = (int)floor(d / INT_MAX);
	theAcc = (int)floor(d - theCnt*(double)INT_MAX);
	return *this;
}

double BigSize::operator /=(const BigSize &sz) const {
	return byted()/sz.byted();
}

double BigSize::byted() const {
	return theCnt*(double)INT_MAX + theAcc;
}

ostream &BigSize::print(ostream &os) const {

	if (theCnt == 0) {
		if (theAcc == 0)
			os << theAcc;
		else
		if (theAcc < 100*1024)
			os << theAcc << "byte";
		else
		if (theAcc < 1024*1024)
			os << theAcc/1024. << "KB";
		else
		if (theAcc < 1024*1024*1024)
			os << theAcc/(1024*1024.) << "MB";
		else
			os << theAcc/(1024*1024*1024.) << "GB";
	} else {
		os << byted()/(1024*1024*1024.) << "GB";
	}

	return os;
}
