
/*
 * $Id: History.h,v 1.4 1999/09/07 17:48:39 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef HISTORY_CLASS_H
#define HISTORY_CLASS_H

#include "Array.h"
#include "gadgets.h"

// History log: a circular buffer; new entries overwrite old ones
// all external coordinates are relative to "now" or last entry
// for simplicity, assume that at most MAX_INT items are inserted
template <class Item>
class History {
	public:
		History(int aCapacity = 0): theLog(aCapacity), theOffset(0) {}

		void capacity(int aCapacity) { Assert(!theOffset); theLog.stretch(aCapacity); }

		int depth() const { return Min(theOffset, theLog.capacity()); }
		Item at(int off) const { return theLog[(theOffset - off) % theLog.capacity()]; }
		Item operator [](int off) const { return at(off); } 

		void insert(Item i) { theLog[++theOffset % theLog.capacity()] = i; }

	protected:
		Array<Item> theLog;
		int theOffset;      // beginning of a buffer
};

#endif
